﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ResetScroll
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ObservableList<Data> data;
        public MainWindow()
        {
            InitializeComponent();
            data = new ObservableList<Data>();

            for (int i = 0; i < 100; i++)
            {
                data.Add(new Data
                {
                    ID = i,
                    Name = "Item " + i,
                });
            }

            DataContext = data;

        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            data.SuspendNotifications = true;
            data.Clear();
            for (int i = 0; i < 200; i++)
            {
                data.Add(new Data
                {
                    ID = i,
                    Name = "Item " + i,
                });
            }
            data.SuspendNotifications = false;

            data.UpdateCollection();
        }
    }

    public class Data
    {
        public int ID { get; set; }
        public string Name { get; set; }
    }

    public class ObservableList<T> : List<T>, INotifyCollectionChanged
    {
        public void Add(T item)
        {
            int index = this.Count - 1;
            base.Add(item);
            OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, item, index));
        }

        public void Remove(T item)
        {
            int index = this.IndexOf(item);
            base.Remove(item);
            OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Remove, item, index));
        }

        public void AddRange(IEnumerable<T> items)
        {
            int index = this.Count - 1;
            base.AddRange(items);
            OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, items.ToList(), index));
        }

        public void Clear()
        {
            base.Clear();
            OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
        }

        private bool _SuspendNotifications;
        public bool SuspendNotifications
        {
            get { return _SuspendNotifications; }
            set { _SuspendNotifications = value; }
        }
         
        public void UpdateCollection()
        {
            bool temp = SuspendNotifications;
            SuspendNotifications = false;
            OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            SuspendNotifications = temp;
        }

        #region INotifyCollectionChanged Members

        public event NotifyCollectionChangedEventHandler CollectionChanged;

        private void OnCollectionChanged(NotifyCollectionChangedEventArgs args)
        {
            if (CollectionChanged != null && !SuspendNotifications)
                CollectionChanged(this, args);
        }

        #endregion

    }
}
